<?php
    /*
      PHP wrapper for Perlfect Search, 2004-02-16
      
      The wrapper is useful if you want to do any of the following:
      - perform PHP operations of your own before you call Perlfect
        (or afterwards on the $result of the call); just insert your 
        code into this script
      - use PHP code in Perlfect templates
      - use AND, OR, NOT operators for your searches
    
      To use this script, you need to modify      
      1. the search form:
         set this file's URL in the 'action' attribute
      2. conf.pl:
         set $SEARCH_URL to this file's URL
      3. the constants below (if the default values don't fit):
         SEARCH_CMD, SEARCH_HTTP
         UXPERL or WINPERL, depending on your system;
           
    */
    
    
    /*
      Adjust the following constants according to your system settings
      
      ----------------------------------------------------------------
    */
    
    // Path to Perl
    define ("UXPERL", "/usr/bin/perl");         // used if your server is running on Unix/Linux
    define ("WINPERL", "C:/perl/bin/perl");     // used if your server is running on Windows
    
    // File system path to Perlfect search script 
    define ("SEARCH_CMD", "/srv/www/cgi-bin/search-3.31b/search.pl");
    
    // URI of the Perlfect search script
    define ("SEARCH_HTTP", "http://localhost/cgi-bin/search-3.31b/search.pl");
    
    /*
      -----------------------------------------------------------------
    */
    
    
    function platform_is_windows () {
        // Returns true if the server is running on a windows platform
        
        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') { 
            return true; 
        } else { 
            return false; 
        }        
    }
    
    function url_fopen_enabled () {
        // Returns if allow_url_fopen is enabled
        // and local files can be addressed via HTTP.
        
        $retrieved = ini_get("allow_url_fopen");
        if (empty($retrieved) or $retrieved == "" or $retrieved == false or $retrieved == "Off" or $retrieved === 0 or $retrieved == "0") {
            return false;
        } else {
            return true;
        }
    }
    
    function php_version_compliant($minimum_requirement) {
        // Returns true if the PHP version meets the $minimum_requirement
        // Note: version_compare can do the same job but is not available
        // before PHP 4.1.0
        return (strcmp($minimum_requirement, phpversion()) <= 0);
    }
    
    /*
       Setting variables which depend on the system configuration:
       $http_call:      true if search.pl can be called via HTTP
       $search_script:  command line or URI for processing search.pl
    */
    
    $http_call = url_fopen_enabled();
    
    if ($http_call) {
        $search_script = SEARCH_HTTP;
    } else {
        if (platform_is_windows()) {
            $search_script = WINPERL." ".SEARCH_CMD;            
        } else {
            $search_script = UXPERL." ".SEARCH_CMD;
        }            
    }


    /*
       Query:
       Preceding/following spaces and quotes cause problems in the search string 
       and thus are removed, and so is "&"
    */
    $query = trim(stripslashes($_GET['q']));
    
    $quoted = (preg_match ("/^\".*\"$|^\'.*\'$/", $query) or $_GET['quoted']);
    $query = str_replace('"', " ", $query);
    $query = str_replace("'", " ", $query);
    $query = str_replace("&", " ", $query);
    $query = trim($query);
    
    // Remove multiple spaces inside the query string
    $query = preg_replace ("/\s+/i", " ", $query);
    
    
    /*
       Convert AND, OR, NOT to Perlfect syntax (+/-)
    */
    $converted = (false or $_GET['converted']);
    
    $mode = $_GET['mode'];
    $lang = $_GET['lang'];
    
    if ($lang == "de") {
            $and = "UND";
            $or = "ODER";
            $not = "NICHT";
    } else {
            $and = "AND";
            $or = "OR";
            $not = "NOT";
    }
    
    if (!$quoted) {
        
        if (preg_match ("/\s(and|or|not|und|oder|nicht)\s/i", $query)) {
        
            $query = preg_replace ("/(\S+)\s(and|und)\s/i", "+$1 +", $query);
            $query = preg_replace ("/(\++)/", "+", $query);
            
            $query = preg_replace ("/\s(not|nicht)\s/i", " -", $query);
            
            if (preg_match ("/\s(or|oder)\s/i", $query)) {
                $query = preg_replace ("/(\S+)\s(or|oder)\s/i", "$1 ", $query);
                $mode = "any";
            }
            $converted = true;
        }
    }
    
    
    /*
      Assemble GET parameter string for the query
    */
    if (empty ($showurl)) {
        
        $params = "p=".urlencode($_GET['p']);
        $params .= "&lang=".urlencode($lang);
        $params .= "&include=".urlencode($_GET['include']);
        $params .= "&exclude=".urlencode($_GET['exclude']);
        $params .= "&penalty=".urlencode($_GET['penalty']);
        $params .= "&mode=".urlencode($mode);
        $params .= "&q=".urlencode($query);
        
        $code_eval = true;
        
    } else {
        
        $params = "q=".urlencode($query);
        $params .= "&showurl=".urlencode($_GET['showurl']);
            
        $code_eval = false;
    }
    
    
    /*
      Call Perlfect
    */
    
    // Temporarily disable Magic Quotes before accessing Perlfect
    $magicquotes = get_magic_quotes_runtime();
    set_magic_quotes_runtime(0);
    
    // Call search.pl
    // Method depends on url_fopen_wrappers
    // being enabled or not ($http_call).
    if ($http_call) {
        
        // Changes in PHP version 4.3.0 and 4.3.2 mean
        // that you have to use different commands to call
        // Perlfect via HTTP, depending on the PHP version
        if (php_version_compliant("4.3.0")) {
            // file_get_contents is not available before 4.3.0
            $result = file_get_contents("$search_script?$params");
        } else {
            // Using fread will not work reliably in 4.3.2 or later 
            $file = fopen("$search_script?$params", "r");
            $result = fread($file, 100000);
            fclose($file);
        }
        
    } else {

        putenv('REQUEST_METHOD=GET');
        putenv('QUERY_STRING='.$params);

        $result = shell_exec("$search_script");
        
        /* If called this way, Perlfect creates headers 
           which are immediately sent.
           
           That is why, in $result, the header line 
           'Content-Type: text/html' must be removed,
           as it is sent already.  */      
        $result = preg_replace ("|Content-Type: text/html\n\n|", "", $result, 1);
        
    }
    
    set_magic_quotes_runtime($magicquotes);
    
    
    /*
       Output.
       Parse PHP code contained in the output, if necessary.
    */
    
    // PHP code may be contained in the results page (the _list_
    // of search results). 
    // Individual hits, processed for search term highlighting,
    // have already been parsed when accessed from search.pl and
    // only contain HTML.
    if ($code_eval) {
        eval ("?>$result<?php ");
    } else {
        print $result;
    }

?>
